﻿/// ==============================================================================================
/// Namespace   : ScipBe.Common.Geocoding
/// Author      : Stefan Cruysberghs
/// Website     : http://www.scip.be
/// Status      : Open source - Mozilla Public License 1.1
/// ==============================================================================================

namespace ScipBe.Common.Geocoding
{
  /// <summary>
  /// Status code
  /// </summary>
  /// <remarks>
  /// http://code.google.com/apis/maps/documentation/reference.html#GGeoStatusCode
  /// </remarks>
  public enum GeoStatusCode
  {
    /// <summary>
    /// No errors occurred; the address was successfully parsed and its geocode has been returned. 
    /// </summary>
    Success = 200,
    /// <summary>
    /// A directions request could not be successfully parsed. For example, the request may have been rejected if it contained more than the maximum number of waypoints allowed. 
    /// </summary>
    BadRequest = 400,
    /// <summary>
    /// A geocoding or directions request could not be successfully processed, yet the exact reason for the failure is not known.
    /// </summary>
    ServerError = 500,
    /// <summary>
    /// The HTTP q parameter was either missing or had no value. For geocoding requests, this means that an empty address was specified as input. For directions requests, this means that no query was specified in the input. 
    /// </summary>
    MissingQuery = 601,
    /// <summary>
    /// Synonym for MissingQuery. 
    /// </summary>
    MissingAddress = 601,
    /// <summary>
    /// No corresponding geographic location could be found for the specified address. This may be due to the fact that the address is relatively new, or it may be incorrect. 
    /// </summary>
    UnknownAddress = 602,
    /// <summary>
    /// 	 The geocode for the given address or the route for the given directions query cannot be returned due to legal or contractual reasons. 
    /// </summary>
    UnavailableAddress = 603,
    /// <summary>
    /// The GDirections object could not compute directions between the points mentioned in the query. This is usually because there is no route available between the two points, or because we do not have data for routing in that region. 
    /// </summary>
    UnknownDirections = 604,
    /// <summary>
    /// The given key is either invalid or does not match the domain for which it was given. 
    /// </summary>
    BadKey = 610,
    /// <summary>
    /// The given key has gone over the requests limit in the 24 hour period or has submitted too many requests in too short a period of time. If you're sending multiple requests in parallel or in a tight loop, use a timer or pause in your code to make sure you don't send the requests too quickly. 
    /// </summary>
    TooManyQueries = 620
  }

  /// <summary>
  /// Address accuracy
  /// </summary>
  /// <remarks>
  /// http://code.google.com/apis/maps/documentation/reference.html#GGeoAddressAccuracy
  /// </remarks>
  public enum GeoAddressAccuracy
  {
    /// <summary>
    /// Unknown location.
    /// </summary>
    UnknownLocation = 0,
    /// <summary>
    /// Country level accuracy. 
    /// </summary>
    Country = 1,
    /// <summary>
    /// Region (state, province, prefecture, etc.) level accuracy. 
    /// </summary>
    Region = 2,
    /// <summary>
    /// Sub-region (county, municipality, etc.) level accuracy. 
    /// </summary>
    SubRegion = 3,
    /// <summary>
    /// Town (city, village) level accuracy. 
    /// </summary>
    Town = 4,
    /// <summary>
    /// Post code (zip code) level accuracy. 
    /// </summary>
    PostCode = 5,
    /// <summary>
    /// Street level accuracy. 
    /// </summary>
    Street = 6,
    /// <summary>
    /// Intersection level accuracy.
    /// </summary>
    Intersection = 7,
    /// <summary>
    /// Address level accuracy. 
    /// </summary>
    Address = 8,
    /// <summary>
    /// Premise (building name, property name, shopping center, etc.) level accuracy.
    /// </summary>
    Premise = 9
  }

  /// <summary>
  /// Interface for Coordinate
  /// </summary>
  public interface ICoordinate
  {
    /// <summary>
    /// Longitude (λ, lambda)
    /// Longitude degrees range from 0° to 180° and these lines are drawn from pole to pole. 
    /// For historical reasons, the meridian passing the Astronomical Observatory in Greenwich, is the one chosen as zero longitude. 
    /// The longitude degrees are completed with the letter E (east) or W (west).
    /// </summary>
    float Longitude { get; }
    /// <summary>
    /// Latitude (Φ, phi).
    /// Latitude lines are horizontal lines shown running east-to-west on maps. 
    /// Technically, latitude is an angular measurement in degrees ranging from 0° at the equator to 90° at the poles. 
    /// The letter N (north) or S (south) is added after the degrees.
    /// </summary>
    float Latitude { get; }
  }

  /// <summary>
  /// Interface for GeoData
  /// </summary>
  /// <remarks>
  /// See OASIS xAL (Address Language) definition
  /// http://www.oasis-open.org/committees/ciq/download.html
  /// </remarks>
  public interface IGeoData
  {
    /// <summary>
    /// Original search string
    /// </summary>
    string SearchAddress { get; }
    /// <summary>
    /// Status
    /// </summary>
    GeoStatusCode Status { get; }
    /// <summary>
    /// Accuracy
    /// </summary>
    GeoAddressAccuracy Accuracy { get; }
    /// <summary>
    /// Coordinate (longitude & latitude)
    /// </summary>
    ICoordinate Coordinate { get; }
    /// <summary>
    /// Id
    /// </summary>
    string Id { get; }
    /// <summary>
    /// Formated address
    /// </summary>
    string Address { get; }
    /// <summary>
    /// Country name code (ISO 3166-2)
    /// </summary>
    string CountryNameCode { get; }
    /// <summary>
    /// Administrative area name
    /// </summary>
    /// <example>
    /// Province, State, County, Kanton, bundesamt, ...
    /// </example>
    string AdministrativeAreaName { get; }
    /// <summary>
    /// Sub administrative area name
    /// </summary>
    string SubAdministrativeAreaName { get; }
    /// <summary>
    /// Locality Name
    /// </summary>
    /// <example>
    /// Cities, reservation and any other built-up areas.
    /// </example>
    string LocalityName { get; }
    /// <summary>
    /// Dependent locality name
    /// </summary>
    string DependentLocalityName { get; }
    /// <summary>
    /// Thorough fare name
    /// </summary>
    /// <example>
    /// Street, Road, Canal, River, ...
    /// </example>
    string ThoroughfareName { get; }
    /// <summary>
    /// Premise name
    /// </summary>
    /// <example>
    /// Airport, railway station, shop in a shopping centre, apartment in a building, room in a hotel, ...
    /// </example>
    string PremiseName { get; }
    /// <summary>
    /// Postal code number
    /// </summary>
    string PostalCodeNumber { get; }
  }
}
